"""计算变异性指标"""
function variability(df::DataFrame)::Vector
    df = normal(df)
    m, n = size(df)
    variabilities = []
    for j in 1:n
        ave = mean(df[!, j])
        sum_var = 0
        for i in 1:m
            sum_var += (df[i, j] - ave)^2 / (m - 1)
        end
        append!(variabilities, sqrt(sum_var))
    end
    return variabilities
end


"""计算相关系数矩阵"""
function correlation(df::DataFrame)::DataFrame
    df = normal(df)
    _, n = size(df)
    result = Matrix{Float64}(undef, n, n)
    for j in 1:n
        for i in 1:n
            result[i, j] = result[j, i] = cor(df[!, i], df[!, j])
        end
    end
    return DataFrame(result, :auto)
end


"""计算冲突性指标"""
function conflict(df::DataFrame)::Vector
    corr_matrix = correlation(df)
    conflicts = []
    m, n = size(corr_matrix)
    for j in 1:n
        conflict = 0
        for i in 1:m
            conflict += (1 - corr_matrix[i, j])
        end
        append!(conflicts, conflict)
    end
    return conflicts
end


"""计算critic权重"""
function critic(df::DataFrame)::Vector
    information = variability(df) .* conflict(df)
    weight = information ./ sum(information)
    return weight
end


"""计算标准离差法权重"""
function stddev(df::DataFrame)::Vector
    _, n = size(df)
    weights = []
    for j in 1:n
        append!(weights, std(df[!, j]))
    end
    return weights ./ sum(weights)
end


"""计算熵权法权重"""
function ewm(df::DataFrame)::Vector
    data = normal(df)
    m, n = size(data)
    entropies = []
    weights = []
    var_mat = Matrix{Float64}(undef, m, n)
    for j in 1:n
        var_mat[:, j] = data[!, j] ./ sum(data[!, j])
        for i in 1:m
            var_mat[i, j] = (var_mat[i, j] == 0) ? 1 : var_mat[i, j]
        end
        entropy = -(1 / log(m)) * sum(var_mat[:, j] .* log.(var_mat[:, j]))
        append!(entropies, entropy)
    end

    for j in 1:n
        weight = (1 - entropies[j]) / (n - sum(entropies))
        append!(weights, weight)
    end
    return weights
end